# encoding: utf-8
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import with_statement, division

import os
import re
from urllib import unquote
from threading import Thread
import sys

from plistlib import readPlist

class iTunes(object):
    tracks = []
    def __init__(self, xml):
        self.xml = xml
        t = Thread(target=self.parse_xml, args=[])
        t.start()
    
    def __repr__(self):
        return self.xml

    def parse_xml(self):
        self.library = readPlist(self.xml)
        for id in self.library['Tracks']:
            self.library['Tracks'][id]['Location'] = unquote(self.library['Tracks'][id]['Location'].replace('file://localhost/', '/'))
            if self.library['Tracks'][id]['Location'].startswith('//'):
                self.library['Tracks'][id]['Location'] = self.library['Tracks'][id]['Location'][1:]

        self.library['Music Folder'] = unquote(self.library['Music Folder'].replace('file://localhost/', '/'))
        if self.library['Music Folder'].startswith('//'):
            self.library['Music Folder'] = self.library['Music Folder'][1:]
        self.root = self.library['Music Folder']
        self.tracks = self.load_tracks()

    def load_tracks(self):
        tracks = []
        keys = [
            'id', 'name', 'artist', 'album', 'year', 'duration', 'size',
            'sortArtist', 'albumArtist', 'sortAlbumArtist', 'compliation',
        ]
        for t in self.library['Tracks']:
            track = self.library['Tracks'][t]
            if track.get('Kind') in ('MPEG audio file', ) and not track.get('Podcast'):
                item = {}
                for key in keys:
                    item[key] = track.get({
                        'id': 'Track ID',
                        'duration': 'Total Time',
                        }.get(
                            key,
                            re.sub(
                                '^(.)',
                                lambda m: m.groups(0)[0].capitalize(),
                                re.sub('([A-Z])', ' \\1', key)
                            )
                        ), None)
                    if key == 'duration' and item[key]:
                        item[key] = item[key] / 1000
                    if item[key] == None:
                        del item[key]
                tracks.append(item)
        return tracks

    def track(self, track_id):
        track = self.library['Tracks'].get(track_id)
        return track and track['Location'] or None

def path():
    if sys.platform == 'darwin':
        path = os.path.expanduser('~/Music/iTunes/iTunes Music Library.xml')
    elif sys.platform == 'win32':
        path = os.path.expanduser('~\\Music\\iTunes\\iTunes Music Library.xml')
    else:
        path = None
    return path
